/* ignore unused exports */
// @ts-self-types="../index.v5.7.d.ts"

export { Float16Array, isFloat16Array } from "./Float16Array.mjs";
export { isTypedArray } from "./isTypedArray.mjs";
export { getFloat16, setFloat16 } from "./DataView.mjs";
export { f16round, f16round as hfround } from "./f16round.mjs";
