"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.f16round = f16round;
var _converter = require("./_util/converter.cjs");
function f16round(x) {
  return (0, _converter.roundToFloat16)(x);
}