import { roundToFloat16 } from "./_util/converter.mjs";

/**
 * returns the nearest half-precision float representation of a number
 * @param {number} x
 * @returns {number}
 */
export function f16round(x) {
  return roundToFloat16(x);
}
